 MODULE polyhedra
!
!S find_polyhedra(atom,legm,poly,npoly)    Utilizza la connettivita' per cercare dei poliedri
!F function numpoly(poly)                  Il numero di poliedri
!S polyhedra_delete(poly,vpos)             Elimina poliedri indicati nel vettore vpos
!S copia_polyhedra(poly1,poly2)            Copia poly2 in poly1
!S update_polyhedra(poly,iorder)           Aggiorna i poliedri se il modello e' cambiato
!F number_of_faces(poly) result(nface)     Numero di facce del poliedro
!F is_in_poly(poly,kat) result(okpol)      L'atomo kat e' coinvolto in qualche poliedro
!S search_in_poly(poly,kat,pvet,np)        Restituisci in pvet tutti gli np poliedri che contengono l'atom kat
!S get_triangles(triang,npoint)            Calcola tutti i possibili triangoli 
!S print_poly(kpr,poly)                    Print polyhedra
!S reallocate_poly(vetr,n,savevet)         Rialloca ad n un vettore di tipo polyhedron_type

 implicit none

 integer, parameter :: NMAX_FACE_POINTS=4
 type face_type
   integer, dimension(NMAX_FACE_POINTS) :: fp
   integer                              :: n
 end type face_type

 type polyhedron_type
   integer                       :: nvert = 0     ! numero di vertici del poliedro
   integer                       :: xc    = 0     ! centro del poliedro
   integer, dimension(6)         :: vert          ! vertices of polyhedron
   type(face_type), dimension(8) :: face          ! facce triangolari del poliedro
   real                          :: d_index = 0.0 ! distortion index
   real                          :: vol = 0.0     ! volume of polyhedron
 contains

   procedure :: number_of_faces
   procedure :: dindex
   procedure :: volume
 end type polyhedron_type

 private is_in_poly_s, is_in_poly_v
 interface is_in_poly
   module procedure is_in_poly_s, is_in_poly_v
 end interface

 CONTAINS

   integer function number_of_faces(poly)
   class(polyhedron_type), intent(in) :: poly
   select case (poly%nvert)
      case (4)
        number_of_faces = 4
      case (5)
        number_of_faces = 5
      case (6)
        number_of_faces = 8
   end select
   end function number_of_faces

!--------------------------------------------------------------------------------------------------

   real function dindex(poly,atom,cell)
   USE atom_basic
   USE unit_cell
   USE cgeom
   class(polyhedron_type), intent(in)        :: poly
   type(atom_type), dimension(:), intent(in) :: atom
   type(cell_type), intent(in)               :: cell
   real, dimension(poly%nvert)               :: dis
   integer                                   :: i
   real                                      :: dism
!
   do i=1,poly%nvert
      dis(i) = distanzaC(atom(poly%xc)%xc,atom(poly%vert(i))%xc,cell%get_g())
   enddo
   dism = sum(dis)/poly%nvert
   dindex = sum(abs(dis-dism))/(poly%nvert*dism)
!
   end function dindex

!--------------------------------------------------------------------------------------------------

   real function volume(poly,atom,cell)
   USE atom_basic
   USE unit_cell
   USE atom_type_util
   class(polyhedron_type), intent(in)        :: poly
   type(atom_type), dimension(:), intent(in) :: atom
   type(cell_type), intent(in)               :: cell
   real, dimension(3)                        :: x1,x2,x3,x4
   integer                                   :: i
!
   select case (poly%nvert)
      case (4)   ! tetrahedron
        x1 = matmul(cell%get_ortom(),atom(poly%vert(1))%xc)
        x2 = matmul(cell%get_ortom(),atom(poly%vert(2))%xc)
        x3 = matmul(cell%get_ortom(),atom(poly%vert(3))%xc)
        x4 = matmul(cell%get_ortom(),atom(poly%vert(4))%xc)
        volume = tetrahedron_volume(x1,x2,x3,x4)

      case (5)  ! square pyramid
!
!       For construction the base is face(1) and the vertex is face(2)%fp(1)
!       Volume is computed dividing the pyramid in two tetrahedra
        x1 = matmul(cell%get_ortom(),atom(poly%face(2)%fp(1))%xc)
        x2 = matmul(cell%get_ortom(),atom(poly%face(1)%fp(1))%xc)
        x3 = matmul(cell%get_ortom(),atom(poly%face(1)%fp(2))%xc)
        x4 = matmul(cell%get_ortom(),atom(poly%face(1)%fp(3))%xc)
        volume = tetrahedron_volume(x1,x2,x3,x4)
        x2 = matmul(cell%get_ortom(),atom(poly%face(1)%fp(3))%xc)
        x3 = matmul(cell%get_ortom(),atom(poly%face(1)%fp(4))%xc)
        x4 = matmul(cell%get_ortom(),atom(poly%face(1)%fp(1))%xc)
        volume = volume + tetrahedron_volume(x1,x2,x3,x4)

      case (6)   ! octahedron
        volume = 0
        x1 = matmul(cell%get_ortom(),atom(poly%xc)%xc)
        do i=1,8
           x2 = matmul(cell%get_ortom(),atom(poly%face(i)%fp(1))%xc)
           x3 = matmul(cell%get_ortom(),atom(poly%face(i)%fp(2))%xc)
           x4 = matmul(cell%get_ortom(),atom(poly%face(i)%fp(3))%xc)
           volume = volume + tetrahedron_volume(x1,x2,x3,x4)
        enddo
   end select
!
   end function volume

!--------------------------------------------------------------------------------------------------

   subroutine find_polyhedra_sym(atom,legm,cell,spg,atoms,legms,poly,npoly,lsym)
!
!  Utilizza la connettivita' per cercare dei poliedri
!
   USE atom_type_util
   USE connect_mod
   USE unit_cell
   USE cgeom
   USE spginfom
   type(atom_type), dimension(:), allocatable, intent(in)               :: atom
   type(bond_type), dimension(:), allocatable, intent(in)               :: legm
   type(cell_type), intent(in)                                          :: cell
   type(spaceg_type), intent(in)                                        :: spg
   type(atom_type), dimension(:), allocatable, intent(inout)            :: atoms
   type(bond_type), dimension(:), allocatable, intent(inout)            :: legms
   type(polyhedron_type), dimension(:), allocatable, intent(inout)      :: poly
   integer, intent(out)                                                 :: npoly
   logical, intent(in)                                                  :: lsym
   integer                                                              :: nat0,nleg0
   integer                                                              :: i,j
   integer, dimension(:), allocatable                                   :: iorder
   integer                                                              :: natsym
   logical                                                              :: lrem
   integer                                                              :: natadd
   !real                                                                 :: init_time,end_time
!
   !call cpu_time(init_time)
   if (.not. lsym) then     ! non e' attiva la simmetria
       call init_for_symm(atom,legm,atoms,legms)
   endif
   nat0 = numatoms(atoms)
   if (nat0 == 0) then ! possible if apply symmetry with 'all atoms fit'
       call reallocate_poly(poly,0)  ! delete all polyhedra if present
       return
   endif
   nleg0 = numbonds(legms)
!
!  Prova ad espandere i contatti
   call expand_contacts(atom,cell,spg,atoms,legms,.false.)
     !write(0,*)'NUMAT=',numatoms(atoms),nat0
!
!  Cerca poliedri
   call find_polyhedra(atoms,legms,cell,poly,npoly)
!
!  Elimina gli atomi aggiunti che non sono coinvolti in poliedri
   natsym = numatoms(atoms)
   natadd = natsym - nat0
   if (npoly > 0) then
       if (natadd > 0) then
           lrem = .false.
           do i=nat0+1,natsym
              do j=1,npoly
                 if (.not.is_in_poly(poly,i)) then
                     !atoms(i)%nz = -999   ! marca l'atomo
                     call atoms(i)%set_as_deleted()
                     !!!!infos(i)%oper = 0    ! marca infos
                     lrem = .true.
                 endif
              enddo
           enddo
           if (lrem) then  ! c'e' qualche atomo da rimuovere?
               allocate(iorder(natsym))
               call remove_deleted(atoms,legm=legms,iord=iorder)
               call update_polyhedra(poly,iorder)
           endif
       endif
   else
!
!      Se non ci sono poliedri, ripristina tutto
       if (natadd > 0) then
           call resize_atoms(atoms,nat0)
           call resize_bonds(legms,nleg0)
       endif
   endif
     !write(0,*)'NUMAT=',numatoms(atoms)
   !call print_poly(0,poly,atoms,cell)
   !call cpu_time(end_time)
   !write(0,*)'time=',end_time-init_time
!
   end subroutine find_polyhedra_sym
 
!--------------------------------------------------------------------------------------------------

   subroutine find_polyhedra(atom,legm,cell,poly,npoly)
!
!  Utilizza la connettivita' per cercare dei poliedri
!
   USE atom_type_util
   USE connect_mod
   USE unit_cell
   USE cgeom
   USE nr
   USE arrayutil
   type(atom_type), dimension(:), allocatable, intent(in)          :: atom
   type(bond_type), dimension(:), allocatable, intent(in)          :: legm
   type(cell_type), intent(in)                                     :: cell
   type(polyhedron_type), dimension(:), allocatable, intent(inout) :: poly
   integer, intent(out)                                            :: npoly
   type(atom_type), dimension(:), allocatable                      :: atomc
   integer                                                         :: natom
   integer                                                         :: nleg
   type(container_type), dimension(:), allocatable                :: conn
   integer                                                         :: i,j,   k
   integer                                                         :: npolmax
   type(face_type), dimension(4)                               :: triangt
   type(face_type), dimension(20)                              :: triango
   type(face_type), dimension(5)                               :: squares
   integer, dimension(4)                                           :: na
   real, dimension(20)                                             :: dist
   integer, dimension(20)                                          :: iord
   integer                                                         :: code, tpos, ntri
   real, dimension(4)                                              :: vp
   integer, dimension(6)                                           :: ntriang
   integer, dimension(1)                                           :: kmin
   real, dimension(3)                                              :: ang
   integer, dimension(4)                                           :: quadr
   integer                                                         :: kvert
!
   npoly = 0
   natom = numatoms(atom)
   nleg = numbonds(legm)
!
   if (nleg > 0) then
!
!      genera permutazioni 
       call get_faces(triangt,4,3)   ! triangles for il tetraedro
       call get_faces(triango,6,3)   ! triangles for l' ottaedro
       call get_faces(squares,5,4)   ! squares for square pyramid
!
!      converti in cartesiane
       call frac_to_cart_copy(atom,atomc,cell%get_ortom())
!
!      Extract conn consideing site
       call bond_to_connect_site(natom,legm,conn,atomc) 
!
!      calcola il numero massimo di poliedri
       !npolmax = count(conn(:)%nat == 4 .or. conn(:)%nat == 6)
       npolmax = count(conn(:)%nat >= 4 .and. conn(:)%nat <= 6)
       call reallocate_poly(poly,npolmax,.false.)  ! alloca poly sul numero massimo
!
       do i=1,natom
          select case (conn(i)%nat)
             case (4)    ! tetrahedro
!
!              Controlla che il poliedro sia chimicamente plausibile
               call check_atoms_poly(i,conn(i)%pos(:conn(i)%nat),atomc,cell,code)
               if (code  == 0) then  ! il poliedro e' buono se code = 0
!
!                  Aggiungi nuovo poliedro
                   npoly = npoly + 1
                   poly(npoly)%nvert = conn(i)%nat
                   poly(npoly)%xc = i
                   poly(npoly)%vert(:4) = conn(i)%pos(:4)
                   do j=1,4
                      poly(npoly)%face(j)%fp(1:3) = conn(i)%pos(triangt(j)%fp(1:3))
                      poly(npoly)%face(j)%n = 3
                   enddo
!
!                  Don't consider atoms that are equivalent to center (e.g. diamond.cif)
                   do j=1,conn(i)%nat
                      if (atom(conn(i)%pos(j))%asym == atom(i)%asym) then
                          conn(conn(i)%pos(j))%nat = 0
                      endif
                   enddo
               endif

             case (5)    ! square pyramid
               call check_atoms_poly(i,conn(i)%pos(:conn(i)%nat),atomc,cell,code)
               if (code == 0) then
!
!                  Find sequence of 4 atoms that minimize distance from central atom i
                   do j=1,size(squares)
!!!TODO: maybe is necessary check planarity of 4 atoms
                      vp(:) = plane_of_atoms(atomc,cell,conn(i)%pos(squares(j)%fp(:4)),.true.)
                      dist(j) = abs(dot_product(vp(:3),atomc(i)%xc)+vp(4))
                      !write(0,*)'DIST=',dist(j),conn(i)%pos(squares(j)%fp(:4))
                   enddo
                   !write(0,*)'MIN=',minloc(dist(:size(squares)))
                   kmin = minloc(dist(:size(squares)))
!
!                  Find quandrilateral searching for the maximum angle
                   na(:4) = conn(i)%pos(squares(kmin(1))%fp(:4))
                   ang(1) = angleC(atomc(na(2))%xc,atomc(na(1))%xc,atomc(na(3))%xc)
                   ang(2) = angleC(atomc(na(3))%xc,atomc(na(1))%xc,atomc(na(4))%xc)
                   ang(3) = angleC(atomc(na(2))%xc,atomc(na(1))%xc,atomc(na(4))%xc)
                   kmin = maxloc(ang)
                   select case (kmin(1))
                     case (1)
                       quadr = [na(1),na(2),na(4),na(3)]
                     case (2)
                       quadr = [na(1),na(3),na(2),na(4)]
                     case (3)
                       quadr = [na(1),na(2),na(3),na(4)]
                   end select
                   !write(0,*)'SQUARE=',quadr
!
!                  Find vertex (atom not in square)
                   do j=1,conn(i)%nat
                      if (all(quadr /= conn(i)%pos(j))) then
                          kvert = conn(i)%pos(j)
                          exit
                      endif
                   enddo
                   !write(0,*)'kvert=',kvert
                   npoly = npoly + 1
                   poly(npoly)%face(1)%fp(1:4) = quadr
                   poly(npoly)%face(1)%n = 4
                   poly(npoly)%nvert = conn(i)%nat
                   poly(npoly)%xc = i
                   poly(npoly)%vert(:5) = conn(i)%pos(:5)
                   do j=2,4
                      poly(npoly)%face(j)%n = 3
                      poly(npoly)%face(j)%fp(1:3) = [kvert,quadr(j-1),quadr(j)]
                   enddo
                   poly(npoly)%face(5)%n = 3
                   poly(npoly)%face(5)%fp(1:3) = [kvert,quadr(4),quadr(1)]
               endif
               
             case (6)    ! ottaedro
!
!              Controlla che il poliedro sia chimicamente plausibile
               call check_atoms_poly(i,conn(i)%pos(:conn(i)%nat),atomc,cell,code)
               if (code  == 0) then  ! il poliedro e' buono se code = 0
!
!                  calcola distanza baricentro-centro poliedro per tutte le facce
                   do j=1,20
                      na(1:3) = conn(i)%pos(triango(j)%fp(1:3))
                      !na(2) = conn(i)%pos(triango(j)%n2)
                      !na(3) = conn(i)%pos(triango(j)%n3)
                      vp(:) = plane3points(atomc(na(1))%xc,atomc(na(2))%xc,atomc(na(3))%xc)
!
!                     after structure cagly_true.cif medium distance from all vertices was chosen
                      dist(j) = 0
                      do k=1,6
                         dist(j) = dist(j) + abs(dot_product(vp(:3),atomc(conn(i)%pos(k))%xc)+vp(4))
                      enddo
                      !dist(j) = abs(dot_product(vp(:3),atomc(i)%xc)+vp(4))
                   enddo
!
!                  scegli le 8 facce che massimizzano la distanza e con atomi coinvolti in 4 facce
                   call indexx(dist,iord)
                   iord(:) = iord(20:1:-1)
!
                   npoly = npoly + 1
                   poly(npoly)%nvert = conn(i)%nat
                   poly(npoly)%xc = i
                   poly(npoly)%vert(:6) = conn(i)%pos(:6)
                   ntriang(:) = 0   ! save the number of triangle for atom (must be == 4)
                   ntri = 0
                   do j=1,20
                      tpos = iord(j)
                      !write(0,*)'valuto tri ',tpos,triango(tpos)%n1,triango(tpos)%n2,triango(tpos)%n3
                      if (ntriang(triango(tpos)%fp(1)) + 1 > 4) cycle
                      if (ntriang(triango(tpos)%fp(2)) + 1 > 4) cycle
                      if (ntriang(triango(tpos)%fp(3)) + 1 > 4) cycle
                      ntriang(triango(tpos)%fp(1)) = ntriang(triango(tpos)%fp(1)) + 1
                      ntriang(triango(tpos)%fp(2)) = ntriang(triango(tpos)%fp(2)) + 1
                      ntriang(triango(tpos)%fp(3)) = ntriang(triango(tpos)%fp(3)) + 1
                      na(1:3) = conn(i)%pos(triango(tpos)%fp(1:3))
                      !na(2) = conn(i)%pos(triango(tpos)%n2)
                      !na(3) = conn(i)%pos(triango(tpos)%n3)
                      ntri = ntri + 1
                      poly(npoly)%face(ntri)%fp(1:3) = na(1:3)
                      poly(npoly)%face(ntri)%n = 3
                      !write(0,*)ntri,'face=',na(:)
                      if (ntri == 8) exit
                   enddo
                   if (ntri /= 8) npoly = npoly - 1  ! Cannot find 8 faces: polyhedra rejected
               endif
          end select
       enddo
   endif
   call reallocate_poly(poly,npoly,.true.) ! ricompatta poly
!
   end subroutine find_polyhedra

!--------------------------------------------------------------------------------------------------

!corr   subroutine find_polyhedra_eq(polyau,npolyau,poly,npoly,infosau,atoms,infos,legms)
!corr   USE atom_type_util
!corr   USE connect_mod
!corr   type(polyhedron_type), dimension(:), allocatable, intent(in) :: polyau
!corr   integer, intent(in)                                          :: npolyau
!corr   type(polyhedron_type), dimension(:), allocatable, intent(inout) :: poly
!corr   integer, intent(inout)                                          :: npoly
!corr   type(symminfo_type), dimension(:), allocatable, intent(in)      :: infosau
!corr   type(atom_type), dimension(:), allocatable, intent(in)          :: atoms
!corr   type(symminfo_type), dimension(:), allocatable, intent(inout)   :: infos
!corr   type(bond_type), dimension(:), allocatable, intent(in)          :: legms
!corr   type(connection_type), dimension(:), allocatable                :: conns
!corr   integer :: natom, npoly0
!corr   integer :: i,j,nf,nc
!corr!
!corr   natom = numatoms(atoms)
!corr   call bond_to_connect(natom,legms,conns)       ! estrai conn da legm
!corr       call reallocate_poly(poly,50,.true.)  !FIXME
!corr   npoly = 0
!corr   do i=1,natom
!corr      do j=1,npolyau
!corr         if (infos(i)%asym == infosau(poly(j)%xc)%asym .and. conns(i)%nat == polyau(j)%nvert) then
!corr          write(0,*)'ATOM ',i,'CONN=',conns(i)%pos
!corr          write(0,*)'ATOM ',i,'ASYM=',infos(conns(i)%pos)%asym
!corr             npoly = npoly + 1
!corr             poly(npoly)%nvert = polyau(j)%nvert
!corr             poly(npoly)%xc = i
!corr             do nf=1,poly(j)%number_of_faces()
!corr                write(0,*)'ASYM faccia:',infosau(polyau(j)%face(nf)%n1)%asym,infosau(polyau(j)%face(nf)%n2)%asym,infosau(polyau(j)%face(nf)%n3)%asym
!corr                do nc=1,conns(i)%nat
!corr                   if (infos(conns(i)%pos(nc))%asym == infosau(polyau(j)%face(nf)%n1)%asym) then
!corr                       poly(npoly)%face(nf)%n1 = conns(i)%pos(nc)
!corr                       cycle
!corr                   endif
!corr                enddo
!corr                do nc=1,conns(i)%nat
!corr                   if (infos(conns(i)%pos(nc))%asym == infosau(polyau(j)%face(nf)%n2)%asym) then
!corr                       poly(npoly)%face(nf)%n2 = conns(i)%pos(nc)
!corr                       cycle
!corr                   endif
!corr                enddo
!corr                do nc=1,conns(i)%nat
!corr                   if (infos(conns(i)%pos(nc))%asym == infosau(polyau(j)%face(nf)%n3)%asym) then
!corr                       poly(npoly)%face(nf)%n3 = conns(i)%pos(nc)
!corr                       cycle
!corr                   endif
!corr                enddo
!corr                write(0,*)'faccia n.',nf,poly(npoly)%face(nf)%n1,poly(npoly)%face(nf)%n2,poly(npoly)%face(nf)%n3
!corr             enddo
!corr          !   if (poly%num_face())
!corr!             poly(npoly) = poly
!corr             write(0,*)'probabile poliedro ',j,' su atomo ',slabnum(atoms(i)%lab,i)
!corr             write(0,*)'at=',conns(i)%nat
!corr         endif
!corr      enddo
!corr   enddo
!corr!
!corr   call reallocate_poly(poly,npoly,.true.) ! ricompatta poly
!corr!
!corr   end subroutine find_polyhedra_eq

!--------------------------------------------------------------------------------------------------

!corr   function poly_eq(poly,infos,xc,vconn)
!corr   type(polyhedron_type), intent(in) :: poly
!corr   type(polyhedron_type)             :: poly_eq
!corr!
!corr!
!corr   end function poly_eq

!--------------------------------------------------------------------------------------------------

   subroutine check_atoms_poly(kat,vet,atomc,cell,code)
   USE atom_type_util
   USE unit_cell
   integer, intent(in)                         :: kat
   integer, dimension(:), intent(in)           :: vet
   type(atom_type), dimension(:), intent(in)   :: atomc  ! coordinate cartesiane
   type(cell_type), intent(in)                 :: cell
   integer, intent(out)                        :: code
!
   call check_atoms(kat,vet,atomc,cell,code)  ! check sugli angoli
   if (code == 0) then                   ! check sulle specie
!
!      non accettare poliedri con non-metallo al centro e metallo ai vertici
!corr       if (.not.is_metal(atomc(kat)) .and. all(is_metal(atomc(vet)))) code = 1
       if (.not.is_metal(atomc(kat)) .and. all_is_metal(atomc(vet))) code = 1
   endif
!
   end subroutine check_atoms_poly

!--------------------------------------------------------------------------------------------------

   integer function numpoly(poly)
!
!  Il numero di poliedri
!
   type(polyhedron_type), dimension(:), allocatable, intent(in) :: poly
!
   if (allocated(poly)) then
       numpoly = size(poly)
   else
       numpoly = 0
   endif
!
   end function numpoly

!--------------------------------------------------------------------------------------------------

   subroutine polyhedra_delete(poly,vpos)
!
!  Elimina poliedri indicati nel vettore vpos
!
   type(polyhedron_type), dimension(:), allocatable, intent(inout) :: poly
   integer, dimension(:), intent(in)                               :: vpos
   integer                                                         :: npoly
   type(polyhedron_type), dimension(:), allocatable                :: polynew
   integer                                                         :: nrem
   integer                                                         :: i
   integer                                                         :: newpoly
!
   npoly = numpoly(poly)
   nrem = size(vpos)
   if (npoly > 0 .and. nrem > 0) then
       newpoly = npoly - nrem
       call reallocate_poly(polynew,newpoly)
       newpoly = 0
       do i=1,npoly
          if (any(vpos == i)) cycle
          newpoly = newpoly + 1
          polynew(newpoly) = poly(i)
       enddo
   endif
   call copia_polyhedra(poly,polynew)
!
   end subroutine polyhedra_delete

!--------------------------------------------------------------------------------------------------

   subroutine copia_polyhedra(poly1,poly2)
!
!  Copia poly2 in poly1
!
   type(polyhedron_type), dimension(:), allocatable, intent(inout) :: poly1
   type(polyhedron_type), dimension(:), allocatable, intent(in)    :: poly2
   integer                                                         :: npoly2
!
   npoly2 = numpoly(poly2)
   call reallocate_poly(poly1,npoly2,savevet=.false.)
   if (npoly2 > 0) then
       poly1(:) = poly2(:)
   endif
!
   end subroutine copia_polyhedra

!--------------------------------------------------------------------------------------------------

   subroutine update_polyhedra(poly,iorder)
!
!  Aggiorna i poliedri se il modello e' cambiato
!  iorder(vecchia poisizione dell'atomo) = nuova posizione
!  iorder(vecchia poisizione dell'atomo) = 0 -> atomo cancellato
!
   type(polyhedron_type), dimension(:), allocatable, intent(inout) :: poly
   integer, dimension(:), intent(in)                               :: iorder
   integer                                                         :: npoly
   integer                                                         :: nat
   integer                                                         :: i,j,k
   integer, dimension(size(poly))                                  :: pvet
   integer                                                         :: np
!
   npoly = numpoly(poly)
   if (npoly > 0) then
!
!      Rimuovi i poliedri che contengono atomi cancellati
       nat = size(iorder)
       do i=1,nat
          if (iorder(i) == 0) then                               ! atomo i eliminato
              call search_in_poly(poly,i,pvet,np)                ! cerca i nel poliedro
              if (np > 0) call polyhedra_delete(poly,pvet(:np))  ! cancella i poliedri in pvet         
          endif
       enddo
!
!      Aggiorna il numero d'ordine degli atomi
       do i=1,numpoly(poly)
          poly(i)%xc = iorder(poly(i)%xc)
          do j=1,poly(i)%number_of_faces()
             do k=1,poly(i)%face(j)%n
                poly(i)%face(j)%fp(k) = iorder(poly(i)%face(j)%fp(k))
             enddo
          enddo
       enddo
   endif
!
   end subroutine update_polyhedra

!--------------------------------------------------------------------------------------------------
  
#if 0
   integer function number_of_faces(poly) result(nface)
!
!  Numero di facce del poliedro
!
   type(polyhedron_type) :: poly
!
   select case (poly%nvert)
      case (4)
        nface = 4
      case (6)
        nface = 8
   end select
!
   end function number_of_faces
#endif

!--------------------------------------------------------------------------------------------------

   logical function is_in_poly_v(poly,kat) result(okpol)
!
!  Is the Atom kat involved in any polyhedra?
!
   type(polyhedron_type), dimension(:), allocatable, intent(in) :: poly
   integer, intent(in)                                          :: kat
   integer                                                      :: i   !,j,k
!
   okpol = .false.
   do i=1,numpoly(poly)
      if (is_in_poly(poly(i),kat)) then
          okpol = .true.
          return
      endif
      !okpol = poly(i)%xc == kat     ! atomo centrale?
      !if (.not.okpol) then          ! ... o un atomo sulle facce
      !    do j=1,poly(i)%number_of_faces()
      !       do k=1,poly(i)%face(j)%n
      !          if (poly(i)%face(j)%fp(k) == kat) then
      !              okpol = .true.
      !              return
      !          endif
      !       enddo
      !       !if (poly(i)%face(j)%n1 == kat .or. poly(i)%face(j)%n2 == kat     &
      !       !    .or. poly(i)%face(j)%n3 == kat) then
      !       !    okpol = .true.
      !       !    exit
      !       !endif
      !    enddo
      !endif
      !if (okpol) exit
   enddo 
!
   end function is_in_poly_v

!--------------------------------------------------------------------------------------------------

   logical function is_in_poly_s(poly,kat) result(okpol)
!
!  Is the Atom kat involved in any polyhedra?
!
   type(polyhedron_type), intent(in) :: poly
   integer, intent(in)               :: kat
   integer                           :: j,k
!
   okpol = poly%xc == kat     ! atomo centrale?
   if (.not.okpol) then       ! ... o un atomo sulle facce
       do j=1,poly%number_of_faces()
          do k=1,poly%face(j)%n
             if (poly%face(j)%fp(k) == kat) then
                 okpol = .true.
                 return
             endif
          enddo
       enddo
   endif
!
   end function is_in_poly_s

!--------------------------------------------------------------------------------------------------

   subroutine search_in_poly(poly,kat,pvet,np)
!
!  Restituisci in pvet tutti gli np poliedri che contengono l'atom kat
!
   type(polyhedron_type), dimension(:), allocatable, intent(in) :: poly
   integer, intent(in)                                          :: kat
   integer, dimension(size(poly)), intent(out)                  :: pvet
   integer, intent(out)                                         :: np
   integer                                                      :: i  !,j
!corr   logical                                                      :: okpol
!
   np = 0
   do i=1,numpoly(poly)
      !okpol = poly(i)%xc == kat     ! atomo centrale?
      !if (.not.okpol) then          ! o un atomo sulle facce
      !    do j=1,poly(i)%number_of_faces()
      !       if (poly(i)%face(j)%n1 == kat .or. poly(i)%face(j)%n2 == kat     &
      !           .or. poly(i)%face(j)%n3 == kat) then
      !           okpol = .true.
      !           exit
      !       endif
      !    enddo
      !endif
      !if (okpol) then
      if (is_in_poly(poly(i),kat)) then
          np = np + 1
          pvet(np) = i
      endif
   enddo
!
   end subroutine search_in_poly

!--------------------------------------------------------------------------------------------------
  
   subroutine get_faces(triang,npoint,k)
!
!  Calcola tutti i possibili triangoli 
!  
   type(face_type), dimension(:), intent(inout) :: triang
   integer, intent(in)                          :: npoint
   integer, intent(in)                          :: k
   logical                                      :: mtc
   integer, dimension(k)                        :: vet
   integer                                      :: nt
!
!corr   k = 3
   mtc = .false.
   nt = 0
   do 
     call nexksb(npoint,k,vet,mtc)
     nt = nt + 1
     triang(nt)%fp(1:k) = vet(1:k)
     triang(nt)%n = k
     !triang(nt)%n1 = vet(1)
     !triang(nt)%n2 = vet(2)
     !triang(nt)%n3 = vet(3)
     if (.not.mtc) exit
   enddo
!
   end subroutine get_faces

!--------------------------------------------------------------------------------------------------
!corr
!corr   subroutine get_squares()
!corr   integer :: npoint
!corr   integer                                      :: k
!corr   logical                                      :: mtc
!corr   integer, dimension(:),allocatable            :: vet
!corr   integer                                      :: nt
!corr   npoint = 5
!corr   k = 4
!corr   allocate(vet(k))
!corr   mtc = .false.
!corr   nt = 0
!corr   do 
!corr     call nexksb(npoint,k,vet,mtc)
!corr     nt = nt + 1
!corr     write(0,*)vet,npoint
!corr     !triang(nt)%fp(1:3) = vet(1:3)
!corr     !triang(nt)%n = 3
!corr     !triang(nt)%n1 = vet(1)
!corr     !triang(nt)%n2 = vet(2)
!corr     !triang(nt)%n3 = vet(3)
!corr     if (.not.mtc) exit
!corr   enddo
!corr
!corr   end subroutine get_squares
!corr
!--------------------------------------------------------------------------------------------------

   subroutine print_poly(kpr,poly,atom,cell)
!
!  Print polyhedra
!
   USE atom_basic
   USE unit_cell
   integer, intent(in)                                          :: kpr
   type(polyhedron_type), dimension(:), allocatable, intent(in) :: poly
   type(atom_type), dimension(:), allocatable                   :: atom
   type(cell_type), intent(in)                                  :: cell
   integer                                                      :: i !,j
   integer :: numau, asym, npoly
   integer, dimension(:), allocatable :: vetd
!
   npoly = numpoly(poly)
   if (npoly == 0) return
!
   write(kpr,'("Tutti i poliedri")')
   do i=1,numpoly(poly)
      write(kpr,'(a,i5,a,i5,a,a,i5)')'nvert=',poly(i)%nvert,' cent=',poly(i)%xc,' at='//trim(atom(poly(i)%xc)%lab),  &
                  ' asym=',atom(poly(i)%xc)%asym
      write(kpr,'(a,*(i5))')'VERT=',poly(i)%vert(:poly(i)%nvert)
      write(kpr,'(2(a,f10.4))')'Volume=',poly(i)%volume(atom,cell),' DI=',poly(i)%dindex(atom,cell)
      !do j=1,poly(i)%number_of_faces()
      !   write(kpr,*)'face ',j,poly(i)%face(j)
      !enddo
   enddo
!
   write(kpr,'("Poliedri distinti")')
   numau = maxval(atom(poly(:)%xc)%asym)
   allocate(vetd(numau),source=0)
   do i=1,numpoly(poly)
      asym = atom(poly(i)%xc)%asym
      if (asym <= numau .and. vetd(asym) == 0) then
          vetd(asym) = i
          write(kpr,'(a,i5,a,i5,a)')'nvert=',poly(i)%nvert,' cent=',poly(i)%xc,' at='//trim(atom(poly(i)%xc)%lab)
      endif
   enddo
!
   end subroutine print_poly

!--------------------------------------------------------------------------------------------------

   subroutine reallocate_poly(vetr,n,savevet)
!
!  Rialloca ad n un vettore di tipo polyhedron_type
!  Se savevet = .true. o non esiste si salva il suo contenuto.
!
   type(polyhedron_type), allocatable, intent(inout) :: vetr(:)
   integer, intent(in)                               :: n
   logical, optional, intent(in)                     :: savevet
   logical                                           :: savev
   integer                                           :: nv
   type(polyhedron_type), allocatable                :: vsav(:)
   integer                                           :: nsav
!
!  se n = 0 (riallocazione a 0): dealloca ed esci
   if (n == 0) then
       if (allocated(vetr)) deallocate(vetr)
       return
   endif
!
   if (.not.allocated(vetr)) then
       allocate(vetr(n))
   else
!
       nv = size(vetr)
       if (present(savevet)) then
           savev = savevet
       else
           savev = .true.
       endif
!
       if (savev) then
!
!          nsav contiene qual è la porzione di vetr da salvare
           select case(nv-n)
             case (1:)       ! compatta x ad n
               nsav = n
             case (:-1)      ! espandi x ad n
               nsav = nv
             case (0)
               return        ! n=nv non fare niente
           end select
           allocate(vsav(n))
           vsav(:nsav) = vetr(:nsav)
           call move_alloc(vsav,vetr)
!
!          salva vetr fino a nsav
!!!corr           allocate(vsav(nsav))
!!!corr           vsav = vetr(:nsav)
!
!          dealloca vetr
!!!corr           deallocate(vetr)
!
!          rialloca vetr su n
!!!corr           allocate(vetr(n))
!
!          ripristina vetr fino a nsav
!!!corr           vetr(:nsav) = vsav(:nsav)
       else
           if (nv /= n) then
               deallocate(vetr)
               allocate(vetr(n))
           endif
       endif
   endif
!
   end subroutine reallocate_poly
   
!--------------------------------------------------------------------------------------------------

   subroutine nexksb(n,k,a,mtc)
!
!  From Chapter 3: Next k-subset of an n-set(p32) - Combinatorial Algorithms
!  Algorithm:Generating k-subset of {1,2,...,n} of an 
!          n-set, in lexicographical order. 
!
   integer a(k) 
   logical mtc
   integer, save :: m2,h
   integer :: k,n
   integer :: i,j
   if(k.gt.0)goto 30
   do 1 i=1,n
 1 a(i)=0
   mtc=.false.
   return
30 if(mtc) go to 40
   m2=0
   h=k
   go to 50
40 if(m2.lt.n-h) h=0
   h=h+1
   m2=a(k+1-h)
50 do 51 j=1,h
51 a(k+j-h)=m2+j
   mtc=a(1).ne.n-k+1
   return
   end subroutine nexksb

!--------------------------------------------------------------------------------------------------

   real function tetrahedron_volume(x1,x2,x3,x4)
!
!  Volume of tetrahedron: |det(v1,v2,v3)|/6
!  where vi are vector representation of the three edges of the tetrahedron
!
   use math_util
   real, dimension(3), intent(in) :: x1,x2,x3,x4
   real, dimension(3,3)           :: v
!
   v(1,:) = x2-x1
   v(2,:) = x3-x1
   v(3,:) = x4-x1
!
   tetrahedron_volume = abs(determinante(v))/6
!
   end function tetrahedron_volume 

 END MODULE polyhedra
