module quantmod

implicit none

private :: quantitative_nobr

contains

   real function sample_density(crystal)
!
!  Density of sample from density and weight fraction of each phase
!
   use crystal_phase
   type(crystal_phase_t), dimension(:), intent(in) :: crystal
!
   sample_density = 1.0/sum(crystal%wt/crystal%dens)
!
   end function sample_density

!------------------------------------------------------------------------------------------------------------------

   real function sample_lac(crystal,lac)
!
!  Compute the mean linear absorption correction (lac) from lac of single phases
!
   use crystal_phase
   type(crystal_phase_t), dimension(:), intent(in) :: crystal
   real, dimension(:), intent(in)                  :: lac
!
   sample_lac = sum(lac*crystal%wt/crystal%dens)*sample_density(crystal)
!
   end function sample_lac
!------------------------------------------------------------------------------------------------------------------

   subroutine brindley_model(crystal,wave)
!
!  Compute the Brindley correction factor (tau). 
!  Approximate weight fractions are required for density of sample and mumed
!
   use crystal_phase
   use elements
   use atom_type_util
   type(crystal_phase_t), dimension(:), intent(inout) :: crystal
   real, intent(in)                                   :: wave
   real, dimension(size(crystal))                     :: lac
   integer                                            :: nph
   real                                               :: mumed,mur
!
   crystal(:)%tau = 1.0
   if (any(crystal%brindley)) then
       do nph=1,size(crystal)
          if (crystal(nph)%brindley) then
              lac(nph) = linear_abs_coeff(crystal(nph)%at,crystal(nph)%dens,wave)    ! mu
          else
              lac(nph) = 0.0
          endif
          crystal(nph)%br_class = brindley_class(crystal(nph)%p_size,lac(nph))
       enddo
       mumed = sample_lac(crystal,lac)
       do nph=1,size(crystal)
          if (crystal(nph)%brindley) then
              mur = (lac(nph) - mumed)*crystal(nph)%p_size/10000.  ! mur=(mu-mumed)*R; cm -> micrometers
              crystal(nph)%tau = get_tau(mur)
             !crystal(nph)%tau = -0.00229 + 2.054*exp(-(mur+0.50356)/0.69525)   ! tau from Lutterotti formula for spherical particles
          endif
       enddo
   endif
!
   end subroutine brindley_model

!------------------------------------------------------------------------------------------------------------------

   integer function brindley_class(p_size,lac)
   real, intent(in) :: p_size,lac
   real             :: muD
!
   muD = lac*p_size/10000
   if (muD > 1) then             
       brindley_class = 4      ! Very coarse powder
   elseif (muD > 0.1) then
       brindley_class = 3      ! Coarse powder
   elseif (muD > 0.01) then
       brindley_class = 2      ! Medium powder, suitable for application of the Brindley model
   elseif (muD > 0.0) then
       brindley_class = 1      ! Fine powder
   else
       brindley_class = 0      ! Undefined
   endif
!
   end function brindley_class

!------------------------------------------------------------------------------------------------------------------

   function br_class_string(crystal) result(str)
   use crystal_phase
   type(crystal_phase_t), intent(in) :: crystal
   character(len=:), allocatable     :: str
   select case (crystal%br_class)
     case (1)
       str = 'fine'
     case (2)
       str = 'medium'
     case (3)
       str = 'coarse'
     case (4)
       str = 'very coarse'
     case default
       str = '-'
   end select
   end function br_class_string

!------------------------------------------------------------------------------------------------------------------

   real function get_tau(mr)  result(tau)
!
!  Compute tau from values of tau vs (mui - mumed)*R evalueted by Brindley solving the integral in
!  Brindley, G. W. (1945). Philos. Mag. 36, 347-369.
!
   USE nr
   real, intent(in)                 :: mr
   real, dimension(29)              :: yd2
   real, dimension(29), parameter   :: mmr = &     ! (mui - mumed)*R
      [-0.50,-0.40,-0.30,-0.20,-0.10,-0.09,-0.08,-0.07,-0.06,-0.05,-0.04,-0.03,-0.02,-0.01,  &
        0.00, 0.01, 0.02, 0.03, 0.04, 0.05, 0.06, 0.07, 0.08, 0.09, 0.10, 0.20, 0.30, 0.40, 0.50]  
   real, dimension(3,29), parameter :: tau_table = reshape([ &       ! 
                                    2.068, 2.036, 2.029,        &
                                    1.813, 1.807, 1.827,        &
                                    1.508, 1.573, 1.585,        &
                                    1.352, 1.353, 1.362,        &
                                    1.159, 1.162, 1.163,        &
                                    1.142, 1.143, 1.144,        &
                                    1.124, 1.125, 1.125,        &
                                    1.107, 1.108, 1.108,        &
                                    1.090, 1.091, 1.091,        &
                                    1.074, 1.073, 1.074,        &
                                    1.059, 1.058, 1.059,        &
                                    1.043, 1.042, 1.042,        &
                                    1.028, 1.027, 1.027,        &
                                    1.014, 1.014, 1.014,        &
                                    1.000, 1.000, 1.000,        &
                                    0.986, 0.986, 0.986,        &
                                    0.972, 0.973, 0.973,        & 
                                    0.959, 0.960, 0.960,        &
                                    0.945, 0.946, 0.947,        &
                                    0.932, 0.933, 0.934,        &
                                    0.918, 0.919, 0.921,        &
                                    0.905, 0.906, 0.908,        &
                                    0.892, 0.893, 0.895,        & 
                                    0.878, 0.879, 0.882,        &
                                    0.865, 0.866, 0.870,        & 
                                    0.742, 0.753, 0.760,        &
                                    0.640, 0.653, 0.671,        & 
                                    0.545, 0.569, 0.587,        &
                                    0.468, 0.496, 0.529], shape(tau_table))
!
   call spline(mmr,tau_table(1,:),0.0,0.0,yd2)
   tau = splint(mmr,tau_table(1,:),yd2,mr)
!
   end function get_tau

!------------------------------------------------------------------------------------------------------------------

   subroutine quantitative(crystal,wave)
!
!  Compute weight fractions from scale factors and tau
!
   use crystal_phase
   type(crystal_phase_t), dimension(:), allocatable, intent(inout) :: crystal
   real, intent(in)                                                :: wave
   integer                                                         :: nph,stdph
   real                                                            :: errstd2
!
   crystal(:)%tau = 1.0
   if (numphase(crystal) == 1) then
       crystal(1)%wt = 1
       crystal(1)%wtsd = 0.0
       return
   endif
   do nph=1,numphase(crystal)  
      call crystal(nph)%set_density()
   enddo
   call quantitative_nobr(crystal)
!
!  Brindley is applied
   if (any(crystal%brindley)) then
       call brindley_model(crystal,wave)
       call quantitative_nobr(crystal)
   endif
!
!  Normalized weight fractions if crystalline standard phase exists
   stdph = 0
   do nph=1,numphase(crystal)
      if (crystal(nph)%is_standard()) then
          stdph = nph
          exit
      endif
   enddo
   if (stdph > 0) then
       crystal%wtn = crystal%wt*crystal(stdph)%ws/crystal(stdph)%wt
       errstd2 = (crystal(stdph)%wtsd / crystal(stdph)%wt)**2
       do nph=1,numphase(crystal)
          crystal(nph)%wtnsd = crystal(stdph)%ws*crystal(nph)%wtn * sqrt((crystal(nph)%wtsd/crystal(nph)%wt)**2 + errstd2)
       enddo
   endif
!
   end subroutine quantitative

!------------------------------------------------------------------------------------------------------------------

   subroutine quantitative_print(crystal,kpr,stype,tagname)
   use crystal_phase
   use strutil
   type(crystal_phase_t), dimension(:), allocatable, intent(in) :: crystal
   integer, intent(in)                                          :: kpr
   character(len=*), intent(in), optional                       :: stype
   character(len=*), intent(in), optional                       :: tagname
   integer                                                      :: nph
   real                                                         :: bias
   logical                                                      :: wknown,is_std
   character(len=1)                                             :: sstype
   character(len=:), allocatable                                :: tname
!
   if (numphase(crystal) == 1) return
!
   if (present(stype)) then
       sstype = stype
   else
       sstype = 'V'
   endif
   if (present(tagname)) then
       tname = tagname
   else
       tname = '?'
   endif
   is_std = any(crystal%is_standard())
   if (index(upper(sstype),'V') > 0) then
       wknown = any(crystal%is_wt_known())
       if (any(crystal%brindley)) then
           write(kpr,'(9x,56("=")/,20x,a)')'Quantitative analysis'
           if (wknown .and. (.not.is_std)) then
               write(kpr,'(10x,a/,9x,76("="))') &
               'Phase                   %w      Esd     Tau    Powder     True    Bias Relative bias'
               do nph=1,numphase(crystal)
                  bias = crystal(nph)%wt-crystal(nph)%wtrue
                  write(kpr,'(10x,a20,3f8.3,3x,a,3f8.3)')crystal(nph)%cr_name,crystal(nph)%wt*100,crystal(nph)%wtsd*100,   &
                  crystal(nph)%tau,br_class_string(crystal(nph)),crystal(nph)%wtrue*100,bias*100,100*(bias)/crystal(nph)%wtrue
               enddo
           else
               write(kpr,'(10x,a/,9x,56("="))')'Phase                   %w      esd     tau    powder'
               do nph=1,numphase(crystal)
                  write(kpr,'(10x,a20,3f8.3,3x,a)')crystal(nph)%cr_name,crystal(nph)%wt*100,crystal(nph)%wtsd*100,   &
                                                  crystal(nph)%tau,br_class_string(crystal(nph))
               enddo
           endif
       else
           write(kpr,'(9x,50("=")/,20x,a)')'Quantitative analysis'
           if (wknown .and. (.not.is_std)) then
               write(kpr,'(10x,a/,9x,70("="))')'Phase                   %w      Esd     True    Bias Relative bias'
               do nph=1,numphase(crystal)
                  bias = crystal(nph)%wt-crystal(nph)%wtrue
                  write(kpr,'(10x,a20,5f8.3)')crystal(nph)%cr_name,crystal(nph)%wt*100,crystal(nph)%wtsd*100,  &
                  crystal(nph)%wtrue*100,bias*100,100*bias/crystal(nph)%wtrue
               enddo
           else
               write(kpr,'(10x,a/,9x,50("="))')'Phase                   %w      esd'
               do nph=1,numphase(crystal)
                  write(kpr,'(10x,a20,f8.3,f8.3)')crystal(nph)%cr_name,crystal(nph)%wt*100,crystal(nph)%wtsd*100
               enddo
           endif
       endif
       if (is_std) then
           write(kpr,'(9x,50("=")/,20x,a)')'Normalized weight fractions'
           if (wknown) then
               write(kpr,'(10x,a/,9x,70("="))')'Phase                   %w      Esd     True    Bias Relative bias'
               do nph=1,numphase(crystal)
                  bias = crystal(nph)%wtn-crystal(nph)%wtrue
                  write(kpr,'(10x,a20,5f8.3)')crystal(nph)%cr_name,crystal(nph)%wtn*100,crystal(nph)%wtnsd*100, &
                  crystal(nph)%wtrue*100,bias*100,100*bias/crystal(nph)%wtrue
               enddo
               bias = sum(crystal(:)%wtrue)-sum(crystal(:)%wtn)
               write(kpr,'(10x,a20,5f8.3)')'Amorphous content   ',(1-sum(crystal(:)%wtn))*100,sum(crystal(:)%wtnsd)*100,   &
               (1-sum(crystal(:)%wtrue))*100,bias*100,100*bias/(1-sum(crystal(:)%wtrue))
           else
               write(kpr,'(10x,a/,9x,50("="))')'Phase                   %w      Esd'
               do nph=1,numphase(crystal)
                  write(kpr,'(10x,a20,f8.3,f8.3)')crystal(nph)%cr_name,crystal(nph)%wtn*100,crystal(nph)%wtnsd*100
               enddo
               write(kpr,'(10x,a20,f8.3,f8.3)')'Amorphous content   ',(1-sum(crystal(:)%wtn))*100,sum(crystal(:)%wtnsd)*100
           endif
       endif
   elseif(index(upper(sstype),'S') > 0) then
       if (is_std) then
           write(kpr,'(a,*(f8.3))')'@@'//trim(tname),crystal(:)%wtn*100,crystal(:)%wtrue*100,   &
           (crystal(:)%wtn-crystal(:)%wtrue)*100,100*(crystal(:)%wtn-crystal(:)%wtrue)/crystal(:)%wtrue
       else
           write(kpr,'(a,*(f8.3))')'@@'//trim(tname),crystal(:)%wt*100,crystal(:)%wtrue*100,   &
           (crystal(:)%wt-crystal(:)%wtrue)*100,100*(crystal(:)%wt-crystal(:)%wtrue)/crystal(:)%wtrue
       endif
   endif
!
   end subroutine quantitative_print

!------------------------------------------------------------------------------------------------------------------

   subroutine quantitative_nobr(crystal)
!
!  Compute weight fractions
!
   use crystal_phase
   type(crystal_phase_t), dimension(:), allocatable, intent(inout) :: crystal
   real, dimension(size(crystal))                                  :: wcryst
   real, dimension(size(crystal))                                  :: zmv
   integer                                                         :: nph
   real                                                            :: wtot
!
   wtot = 0
   do nph=1,numphase(crystal)
      zmv(nph) = crystal(nph)%dens*crystal(nph)%cell%volume()**2 / crystal(nph)%tau
      wcryst(nph) = crystal(nph)%scal * zmv(nph)
      wtot = wtot + wcryst(nph)
   enddo
!
   crystal(:)%wt = wcryst(:)/wtot
   crystal(:)%wtsd = crystal(:)%ssd * (zmv(:)*wtot - zmv(:)**2 * crystal(:)%scal) / wtot**2
!
   end subroutine quantitative_nobr

end module quantmod
